#include "Flock.h"
#include "glPrint.h"
#include <math.h>
#include <queue>
#include <Mmsystem.h>
#include <stdio.h>

//#define PDBG

GLuint Flock::agentDL = 0;
char * fnames[] = {"Align","Cohes","Sep","Avoid","Wander","Seek","Boost","Wind","Drag"};


Flock::Flock()
{
	file = NULL;
	numAgents =0;
	numObstacles = 0;
	agents = updateagents = NULL;
	obstacles = NULL;

	mass = 1; //kg
//	max_velocity = 200; // m/s
	//max_force = 100.0f; // N
	targspeed = 10;

	//initialize the gsl RNG
	gsl_rng_env_setup();
	const gsl_rng_type * T = gsl_rng_default;
	rng = gsl_rng_alloc(T);
	//gsl_rng_set(rng,timeGetTime());
	//viewing vars
	toroidY = 1000;
	toroidX = 1000;
	viewW = toroidX;
	viewH = toroidY;
	panX = toroidX/2-viewW/2;
	panY = toroidY/2-viewW/2;


	//neighbourhood definition
	nradius = 20;
	nangle = 300*(3.14159/180);
	maxa = 150*(3.14159/180.0);
	mina = -150*(3.14159/180.0);


	cellsX = ceilf(toroidX / (2*nradius));
	cellsY = ceilf(toroidY / (2*nradius));

	//init the cells
	cells.resize(cellsX*cellsY);


	//default weights
	memset(alpha,0,sizeof(double)*FLAST);

}

Flock::~Flock()
{
	//free up the rng
	gsl_rng_free(rng);
	free(agents);
	free(updateagents);
}

void Flock::PanView(double x, double y)
{

	panX = max(0, panX+x);
	panX = min(panX, toroidX-viewW);

	panY = max(0, panY+y);
	panY = min(panY, toroidY-viewH);
}

void Flock::ZoomView(double x, double y)
{
	viewW = min(viewW*x,toroidX);
	viewH = min(viewH*y,toroidY);
	PanView(0,0);
}

void Flock::Render()
{


	//pan to the 1st agent
	panX = max(0, agents[0].position.x-viewW/2);
	panX = min(panX, toroidX-viewW);
	panY = max(0, agents[0].position.y -viewH/2);
	panY = min(panY, toroidY-viewH);


	double cellW = toroidX / cellsX;
	double cellH = toroidY / cellsY;

	glPolygonOffset(1, 1);

	ApplyTransform();

	//render the table
	glColor3f(.25,.25,.25);

	glEnable(GL_POLYGON_OFFSET_FILL);

	//draw the floor as a bunch of quads to improve the lighting
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0,1,0);
	int numstrips = 10;

	double stripwidth = 1.0/numstrips;
	double stripheight = 1.0/numstrips;
	for (int strip = 0; strip < numstrips; strip++)
	{
		glVertex3f(-.5+(strip+1)*stripwidth, 0, -.5);
		glVertex3f(-.5+strip*stripwidth, 0, -.5);
		for (int quad = 1; quad < numstrips+1; quad++)
		{
			glVertex3f(-.5+(strip+1)*stripwidth, 0, -.5+quad*stripheight);
			glVertex3f(-.5+strip*stripwidth, 0, -.5+quad*stripheight);
		}
	}
	glEnd();

	glDisable(GL_POLYGON_OFFSET_FILL);


	//look at our current location
	glScalef(1.0/viewW,1.0/viewW,1.0/viewH);
	glTranslatef(-panX-viewW/2,0,-panY-viewH/2);


	//draw the cell lines

	glColor3f(0,0,0);
	glBegin(GL_LINES);
	for (int x = 0;x <= cellsX; x++)
	{
		glVertex3f(x* toroidX/cellsX,0,0);
		glVertex3f(x* toroidX/cellsX,0,toroidY);
	}
	for (int y = 0;y <= cellsY; y++)
	{
		glVertex3f(0,0,y*toroidY/cellsY);
		glVertex3f(toroidY,0,y*toroidY/cellsY);
	}
	glEnd();



	//only draw the agents within the visable cells
	int minCellX = floorf(panX / cellW);
	int minCellY = floorf(panY / cellH);
	int maxCellX = min(ceilf((panX+viewW)/cellW),cellsX);
	int maxCellY = min(ceilf((panY+viewH)/cellH),cellsY);

	for (int cellx = minCellX; cellx < maxCellX;cellx++)
		for (int celly = minCellY; celly < maxCellY;celly++)
		{
			cvtype::iterator itv = &cells.at(cellx+celly*cellsX);

			glColor3f(0,1,0);
			glPushMatrix();
			glTranslatef(cellx*cellW + cellW/2,0,celly*cellH + cellH/2);
			glPrint(GLUT_BITMAP_TIMES_ROMAN_10,"%d",itv->size());
			glPopMatrix();


			for (cstype::iterator it = itv->begin(); it != itv->end();it++)
			{
				int a = *it;
				glPushMatrix();


				glTranslatef(agents[*it].position.x,0,agents[*it].position.y);
				glPushMatrix();
				glRotatef(-agents[*it].angle*(180/3.14159)+90,0,1,0);
				//glRotatef(agents[*it].angle*(180/3.14159)+90,0,1,0);

				if (*it == 0)
				{
					glColor3f(1,0,0);
					glBegin(GL_LINE_LOOP);
					glNormal3f(0,1,0);
					glVertex3f(0,0,0);

					//fix these
					double maxangle = maxa;//(3.0/2.0)*3.14159- (2*3.14159-nangle)/2;
					double minangle = mina;//-(3.14159/2.0)+ (2*3.14159-nangle)/2;
					
					double anglestep = (maxangle - minangle)/32;
					for (double ang = maxangle; ang > minangle; ang-= (maxangle-minangle)/16)
						glVertex3f(nradius*sinf(ang),0,nradius*cosf(ang));
					glVertex3f(0,0,0);
					glEnd();


					glPrint(GLUT_BITMAP_TIMES_ROMAN_24,"%f",agents[*it].objective);

				}
				glColor3f(1,1,1);
				glEnable(GL_LIGHTING);
				glCallList(agentDL);
				glDisable(GL_LIGHTING);

				glPopMatrix();
				double forceAngle = atan2f(agents[*it].force.y,agents[*it].force.x);
				glColor3f(0,1,0);

				double forcemag = sqrtf(agents[*it].force.y*agents[*it].force.y+agents[*it].force.x*agents[*it].force.x);
				glBegin(GL_LINES);
				glVertex3f(0,0,0);
				glVertex3f(forcemag*cosf(forceAngle),0,forcemag*sinf(forceAngle));
				glEnd();


				glPopMatrix();
			}
		}




		for(int i = 0; i < numObstacles;i++)
		{
			glPushMatrix();
			glColor3f(0,1,0);
			glTranslatef(obstacles[i].position.x,0,obstacles[i].position.y);
			glBegin(GL_POLYGON);
			glNormal3f(0,1,0);
			for (int j = 360; j > 0; j-=10)
				glVertex3f(obstacles[i].radius*cosf( j*(3.14159/180.0) ),0,obstacles[i].radius*sinf(j*(3.14159/180.0)));

			glEnd();
			glColor3f(1,0,0);
			glTranslatef(0,1,0);
			glPopMatrix();
		}

		glColor3f(1,0,0);
		glPushMatrix();
		glTranslatef(seekPos.x,0,seekPos.y);
		glBegin(GL_POLYGON);
		glNormal3f(0,1,0);
		for (int j = 360; j > 0; j-=45)
			glVertex3f(2*cosf( j*(3.14159/180.0) ),0,2*sinf(j*(3.14159/180.0)));
		glEnd();

		glPopMatrix();




}
void Flock::SetAlpha(forceenum f, double weight)
{
	alpha[f] = weight;
}
double Flock::GetAlpha(forceenum f)
{
	return alpha[f];
}

double Flock::Step(double etime)
{
	time+= etime;
	//double obpos[2] = {toroidX/2 + 100, toroidY/2};
	//double obrad = 10;
	double flockOb = 0;


	//don't need to calc these every time
	double cellW = toroidX / cellsX;
	double cellH = toroidY / cellsY;
	double sqnradius = nradius*nradius;

	seekPos.x =toroidX/2+200*cos((time/100 )* 2*3.14159);
	seekPos.y =toroidY/2+200*sin((time/100 )* 2*3.14159);

#ifdef PDBG
	printf("Step %d\n",step);
#endif
	
	
	memset(updateagents,0,numAgents*sizeof(agenttype));
	//do the flock caculations on the agents
	for (int i = 0; i < numAgents;i++)
	{
#ifdef PDBG
	printf(" Agent %d\n",i);
#endif

		//update velocity based upon acceleration
		updateagents[i].velocity.x = agents[i].velocity.x+agents[i].acceleration.x *etime;
		updateagents[i].velocity.y = agents[i].velocity.y+agents[i].acceleration.y *etime;


		//figure out speed and angle and save it for later
		updateagents[i].speed = sqrtf(updateagents[i].velocity.x*updateagents[i].velocity.x+updateagents[i].velocity.y*updateagents[i].velocity.y);
		if (updateagents[i].speed > .001)
			updateagents[i].angle = atan2f(updateagents[i].velocity.y,updateagents[i].velocity.x);
		else
			updateagents[i].angle = 0;

		//update position based upon velocity
		updateagents[i].position.x = agents[i].position.x+ agents[i].velocity.x * etime;
		updateagents[i].position.y = agents[i].position.y+ agents[i].velocity.y * etime;

		//wrap around the toroid
		if (updateagents[i].position.x >= toroidX)
			updateagents[i].position.x -= toroidX;

		if (updateagents[i].position.y >= toroidY)
			updateagents[i].position.y -= toroidY;

		if (updateagents[i].position.x < 0)
			updateagents[i].position.x += toroidX;

		if (updateagents[i].position.y < 0)
			updateagents[i].position.y += toroidY;


		for (int j = 0; j < FLAST;j++) //derivative wrt alpha_j
		{

			updateagents[i].d_velocity[j].x = agents[i].d_velocity[j].x + agents[i].d_acceleration[j].x*etime;
			updateagents[i].d_velocity[j].y = agents[i].d_velocity[j].y + agents[i].d_acceleration[j].y*etime;


			updateagents[i].d_position[j].x = agents[i].d_position[j].x + agents[i].d_velocity[j].x*etime;
			updateagents[i].d_position[j].y = agents[i].d_position[j].y + agents[i].d_velocity[j].y*etime;

#ifdef PDBG
	if (alpha[j] != 0)
	{
		printf("  dvx/d%s %f\n",fnames[j],updateagents[i].d_velocity[j].x);
		printf("  dvy/d%s %f\n",fnames[j],updateagents[i].d_velocity[j].y);
		printf("  dpx/d%s %f\n",fnames[j],updateagents[i].d_position[j].x);
		printf("  dpy/d%s %f\n",fnames[j],updateagents[i].d_position[j].y);
	}
#endif

		}
	}



	//calculate forces
	for (int i = 0; i < numAgents;i++)
	{
		//determine (approximately) which cells are currently touched by our neighbourhood
		int minCellX = max(floorf((agents[i].position.x-nradius) / cellW),0);
		int minCellY = max(floorf((agents[i].position.y-nradius) / cellH),0);
		int maxCellX = min(ceilf((agents[i].position.x+nradius)/cellW),cellsX);
		int maxCellY = min(ceilf((agents[i].position.y+nradius)/cellH),cellsY);

		//this will keep track of how many neighbours we have
		int neighbours = 0;

		updateagents[i].nFn = 0;



		//loop though all possible neighbours
		for (int cellx = minCellX; cellx < maxCellX;cellx++)
			for (int celly = minCellY; celly < maxCellY;celly++)
			{
				cvtype::iterator itv = &cells.at(cellx+celly*cellsX);
				for (cstype::iterator it = itv->begin(); it != itv->end();it++)
				{
					//we are not our own neighbour
					if (*it == i)
						continue;

					int targ = *it;
					//offset to neighbour
					double dx = updateagents[*it].position.x - updateagents[i].position.x;
					double dy = updateagents[*it].position.y - updateagents[i].position.y;
					double sqdistance = (dx*dx+dy*dy);

					//possible neighbour in range. . . 
					if ((sqdistance < sqnradius) && (sqdistance > 0))
					{

						//do the neighbourhood function calculation
						double distance = sqrtf(sqdistance);
						double angle;
						if ((agents[i].speed < .01) || (distance < .0001))
							angle = 0;
						else
						{
							angle = acosf(	(updateagents[i].velocity.x*(updateagents[*it].position.x - updateagents[i].position.x)+
										updateagents[i].velocity.y*(updateagents[*it].position.y - updateagents[i].position.y)) 
										/ ( updateagents[i].speed*distance )  );
						}

						
						updateagents[i].view = 0;

						//derivatives of angle and distance
						double ddist[FLAST];
						for (int j = 0; j < FLAST;j++)
						{
							if (distance > .0001)
								ddist[j] = (dx*(updateagents[i].d_position[j].x - updateagents[*it].d_position[j].x)+dy*(updateagents[i].d_position[j].y - updateagents[*it].d_position[j].y))/ distance;
							else
								ddist[j] = 0;
						}


						double dangle[FLAST];
						for (int j = 0; j < FLAST;j++)
						{
							//intermediates
							double vx = updateagents[i].velocity.x*dx;
							double vy = updateagents[i].velocity.y*dy;
							double sqs = updateagents[i].velocity.x*updateagents[i].velocity.x+updateagents[i].velocity.y*updateagents[i].velocity.y;
							double sqd = dx*dx*dy*dy;
							
							//dintermediates
							double dvx = updateagents[i].velocity.x*(updateagents[*it].d_position[j].x - updateagents[i].d_position[j].x) + updateagents[i].d_velocity[i].x * dx;
							double dvy = updateagents[i].velocity.y*(updateagents[*it].d_position[j].y - updateagents[i].d_position[j].y) + updateagents[i].d_velocity[i].y * dy;
							double dsqs = 2*updateagents[i].velocity.x*updateagents[i].d_velocity[j].x + 2*updateagents[i].velocity.y*updateagents[i].d_velocity[j].y;
							double dsqd = 2*dx*(updateagents[*it].d_position[j].x - updateagents[i].d_position[j].x) + 2*dy*(updateagents[*it].d_position[j].y - updateagents[i].d_position[j].y);

							dangle[j] = 0;
							if (distance*updateagents[i].speed*sqs*sqd > .00001)
							{
								double denom = sqrt(1-(vx+vy)*(vx+vy)/(agents[i].speed*distance));
								if (denom  > .00001)
									dangle[j] = -((dvx+dvy)/(distance*updateagents[i].speed)
											-((vx+vy)/2.0)*(dsqs/(sqs*updateagents[i].speed*distance) + dsqd/(sqd*distance*updateagents[i].speed)))
											/ denom;

							}
						}
	
						
						double beta = 100;
						double maxdist = nradius;
						double na = 1.0 / (1 + exp(beta*((mina-maxa))+exp(beta*(angle-maxa))+exp(beta*(mina-angle))));
						double nd = 1.0 / (1 +exp(beta*(distance-maxdist)));


						//neighbourhood fn
						updateagents[i].nFn += na*nd;


						//derivative of neighbourhood fn
						double dnFn[FLAST];
						for (int j = 0; j < FLAST;j++)
						{
							//getting denormals
							double dna = -dangle[j]*(exp(beta*(angle - maxa))-exp(beta*(mina-angle)))*na*na*beta;
							double dnd = ddist[j]*exp(distance-maxdist)*nd*nd;
							dnFn[j] = dna*nd+dnd*na;
						}


						//make sure we have at least one neighbour
						if (na*nd < .001)
							continue;

						//update the flocking forces
						
						//separation

						double smag = 1 - distance*distance/(nradius*nradius/16 +distance*distance);
						updateagents[i].forces[Fseparation].x += -(dx/distance)*smag;
						updateagents[i].forces[Fseparation].y +=  -(dy/distance)*smag;


						//alignment
						if (agents[*it].speed > .0001)
						{
							updateagents[i].forces[Falignment].x += updateagents[*it].velocity.x/updateagents[*it].speed;
							updateagents[i].forces[Falignment].y += updateagents[*it].velocity.y/updateagents[*it].speed;
						}

						//cohesion
						if (distance > .001)
						{
							updateagents[i].forces[Fcohesion].x += dx/distance;
							updateagents[i].forces[Fcohesion].y += dy/distance;
						}

						//now do derivatives of these forces
						for (int j = 0; j < FLAST;j++)
						{

							if (distance > .001)
							{
								double ddist = ((updateagents[i].position.x - updateagents[*it].position.x)*(updateagents[i].d_position[j].x - updateagents[*it].d_position[j].x)
											   +(updateagents[i].position.y - updateagents[*it].position.y)*(updateagents[i].d_position[j].y - updateagents[*it].d_position[j].y)) / distance;
								double denom = (nradius*nradius/16+distance*distance);
								double dmag = (2*distance*ddist/denom)*(distance*distance/(denom*denom) - 1);
								double ddirx = (updateagents[i].d_position[j].x - updateagents[*it].d_position[j].x)/distance
												- (updateagents[i].position.x - updateagents[*it].position.x)*ddist/(distance*distance);
								
								double ddiry = (updateagents[i].d_position[j].y - updateagents[*it].d_position[j].y)/distance
												- (updateagents[i].position.y - updateagents[*it].position.y)*ddist/(distance*distance);

								updateagents[i].d_forces[Fseparation][j].x = -(dmag*(updateagents[i].position.x - updateagents[*it].position.x)/distance + smag*ddirx);
								updateagents[i].d_forces[Fseparation][j].y = -(dmag*(updateagents[i].position.y - updateagents[*it].position.y)/distance + smag*ddiry);

								if (distance > .001)
								{
									updateagents[i].d_forces[Fcohesion][j].x = (agents[*it].d_position[j].x-agents[i].d_position[j].x)/distance - ddist*dx/(distance*distance);
									updateagents[i].d_forces[Fcohesion][j].y = (agents[*it].d_position[j].y-agents[i].d_position[j].y)/distance - ddist*dy/(distance*distance);
								}
							}
							else
							{
								//printf("too close!");
							}

						}

						//do the neighbour-based part of the objective function here, since we've already done the legwork
						float targdist = 5;						
						updateagents[i].objectiveN += (1 - distance*distance/(targdist*targdist+distance*distance));
						
						for (int j = 0; j < FLAST;j++)
						{
							updateagents[i].d_objectiveN[j] -= (2*distance*ddist[j]*(distance*distance/((targdist*targdist+distance*distance)*(targdist*targdist+distance*distance)) - 1.0/(targdist*targdist+distance*distance)));
						}
					}
				}
			}


			//complete the neightbour force calculations by doing the averaging division
			if (neighbours)
			{
				//updateagents[i].forces[Falignment].x  /= agents[i].nFn;
				//updateagents[i].forces[Falignment].y  /= agents[i].nFn;
			}

			updateagents[i].forces[Fcohesion].x  /= neighbours+1;//(agents[i].nFn+1);
			updateagents[i].forces[Fcohesion].y  /= neighbours+1;//(agents[i].nFn+1);



			//loop through all obstacles to calculate avoidance force
/*
			int cellX = agents[i].cell % cellsX;
			int cellY = agents[i].cell / cellsX;

			for(int j = 0; j < numObstacles;j++)
			{
				if ((cellX > obstacles[j].maxcellX) ||
					(cellX < obstacles[j].mincellX) ||
					(cellY > obstacles[j].maxcellY) ||
					(cellY < obstacles[j].mincellY) )
					continue;

				//dot product of unit velocity vector with vector to obstacle centre
				double closestProj = (obstacles[j].position.x - agents[i].position.x) * agents[i].velocity.x / agents[i].speed +
					(obstacles[j].position.y - agents[i].position.y) * agents[i].velocity.y / agents[i].speed;

				double distsq = (obstacles[j].position.x - agents[i].position.x)*(obstacles[j].position.x - agents[i].position.x)+
					(obstacles[j].position.y - agents[i].position.y)*(obstacles[j].position.y - agents[i].position.y);

				//make sure that the obstacle is in our neighbourhood
				if ((closestProj <= 0) || (distsq > (nradius+obstacles[j].radius)*(nradius+obstacles[j].radius)))
					continue;

				//point of closest approach
				double cDx = agents[i].position.x+(agents[i].velocity.x/agents[i].speed)*closestProj;
				double cDy = agents[i].position.y+(agents[i].velocity.y/agents[i].speed)*closestProj;
				double cdistsq = (obstacles[j].position.x - cDx)*(obstacles[j].position.x - cDx)+
					(obstacles[j].position.y - cDy)*(obstacles[j].position.y - cDy);


				//check for a potential collision
				if (cdistsq > (obstacles[j].radius+1.5)*(obstacles[j].radius+1.5))
					continue;

				//we have a potential collision now, so move!
				double left = agents[i].angle + 3.14159/2.0;
				double dot = cosf(left)* (obstacles[j].position.x - agents[i].position.x) + sinf(left)* (obstacles[j].position.y - agents[i].position.y);
				double str = (max(-closestProj+2*nradius,0)*max(-closestProj+2*nradius,0));
				//double str = max_force* ((nradius+obstacles[j].radius-closestProj)/closestProj)*((nradius+obstacles[j].radius-closestProj)/closestProj);

				if (dot < 0)
				{
					updateagents[i].forces[Favoidance].x += cosf(left)*str;
					updateagents[i].forces[Favoidance].y += sinf(left)*str;
				}
				else
				{
					updateagents[i].forces[Favoidance].x += -cosf(left)*str;
					updateagents[i].forces[Favoidance].y += -sinf(left)*str;
				}

			}
*/


			//seek force
			double seekdx = seekPos.x - updateagents[i].position.x;
			double seekdy = seekPos.y - updateagents[i].position.y;
			double dseek = sqrtf( seekdx*seekdx+seekdy*seekdy);
			
			double seekmag = dseek*dseek/(100*100 + dseek*dseek);
			if (dseek > .0001)
			{
				updateagents[i].forces[Fseek].x += (seekdx/dseek)*seekmag;
				updateagents[i].forces[Fseek].y +=  (seekdy/dseek)*seekmag;
			}




			//wander force
			updateagents[i].forces[Fwander].x = gsl_ran_gaussian(rng, 1);
			updateagents[i].forces[Fwander].y = gsl_ran_gaussian(rng, 1);

			//drag force
			updateagents[i].forces[Fdrag].x = -updateagents[i].velocity.x;
			updateagents[i].forces[Fdrag].y = -updateagents[i].velocity.y;


			//force derivatives
			if (learning)
			{
				for (int k = 0; k < FLAST; k++)
				{
						
					//seek
					
					double ddseek = 0;
					if (dseek > .01)
					{
						ddseek = (seekdx*agents[i].d_position[k].x+seekdy*agents[i].d_position[k].y)/dseek;
					
						double ddmag = (2*dseek*ddseek/(100*100 + dseek*dseek))*(1.0 - dseek*dseek*dseek/(100*100+dseek*dseek));
						updateagents[i].d_forces[Fseek][k].x = (updateagents[i].d_position[k].x/dseek - seekdx*ddseek/(dseek*dseek))*seekmag
															+ ddmag*seekdx/dseek;
						updateagents[i].d_forces[Fseek][k].y = (updateagents[i].d_position[k].y/dseek - seekdy*ddseek/(dseek*dseek))*seekmag
															+ ddmag*seekdy/dseek;
					}


					//drag
					updateagents[i].d_forces[Fdrag][k].x = -updateagents[i].d_velocity[k].x;
					updateagents[i].d_forces[Fdrag][k].y = -updateagents[i].d_velocity[k].y;


					//boost
//					updateagents[i].d_forces[Fboost][k].x = -updateagents[i].d_velocity[k].x;
//					updateagents[i].d_forces[Fboost][k].y = -updateagents[i].d_velocity[k].y;

				}
			}

			
			//now add up all of the forces, with the alpha vector as a weight

			updateagents[i].force.x = 0; 
			updateagents[i].force.y = 0; 
			for (int j = 0; j < FLAST;j++)
			{
				updateagents[i].force.x += alpha[j] * updateagents[i].forces[j].x;
				updateagents[i].force.y += alpha[j] * updateagents[i].forces[j].y;
			}

			
			//randomness!
			//updateagents[i].force.x += gsl_ran_gaussian(rng, .5);
			//updateagents[i].force.y += gsl_ran_gaussian(rng, .5);

			updateagents[i].acceleration.x = updateagents[i].force.x/mass;
			updateagents[i].acceleration.y = updateagents[i].force.y/mass;
			
			if (learning)
			{
				//calculate the derivative of the summed forces
				for (j = 0; j < FLAST;j++) //derivative wrt alpha_j
				{

					updateagents[i].d_force[j].x = updateagents[i].forces[j].x;
					updateagents[i].d_force[j].y = updateagents[i].forces[j].y;

					for (int k=0;k<FLAST;k++)
					{
						updateagents[i].d_force[j].x += alpha[k]*updateagents[i].d_forces[k][j].x;
						updateagents[i].d_force[j].x += alpha[k]*updateagents[i].d_forces[k][j].y;
					}
					
				}



				//normalize the force derivatives, or they get too big!

				/*
				for (j = 0; j < FLAST;j++)
				{

					double ntot = sqrt(updateagents[i].d_force[j].x*updateagents[i].d_force[j].x+updateagents[i].d_force[j].y*updateagents[i].d_force[j].y);
					if (ntot > .0001)
					{
						updateagents[i].d_force[j].x /= ntot;
						updateagents[i].d_force[j].y /= ntot;
					}

				}

				*/

				//calculate the derivative of acceleration
				for (j = 0; j < FLAST;j++) //derivative wrt alpha_j
				{
					updateagents[i].d_acceleration[j].x = updateagents[i].d_force[j].x/mass;
					updateagents[i].d_acceleration[j].y = updateagents[i].d_force[j].y/mass;
				}
			}


	}

	//put agents into their new cells, if neccessary
	for (i = 0 ; i < numAgents;i++)
	{
		updateagents[i].cell = floorf(updateagents[i].position.x / cellW) + cellsX* floorf(updateagents[i].position.y / cellH);
		if (updateagents[i].cell != agents[i].cell)
		{
			cells.at(agents[i].cell).erase(i);
			cells.at(updateagents[i].cell).insert(i);
		}
	}


	//calc the objective function

	if (learning)
	{
		memset(d_fobj,0,FLAST*sizeof(double));
		fobj = 0;
		for (int i = 0; i < numAgents;i++)
		{
			updateagents[i].objective = updateagents[i].objectiveN;

			//speed penalty
			updateagents[i].objective += 10*updateagents[i].speed*updateagents[i].speed;
			
			//acceleration penalty
			updateagents[i].objective += updateagents[i].acceleration.x*updateagents[i].acceleration.x+updateagents[i].acceleration.y*updateagents[i].acceleration.y;

			//seek penalty
			double dseeksq = ( (updateagents[i].position.x - seekPos.x)*(updateagents[i].position.x - seekPos.x)
							+ (updateagents[i].position.y - seekPos.y)*(updateagents[i].position.y - seekPos.y));
			
			updateagents[i].objective += dseeksq;


			//add to flock objective function
			fobj += updateagents[i].objective*etime;
			
			for (int k = 0; k < FLAST; k++)
			{
				updateagents[i].d_objective[k] = updateagents[i].d_objectiveN[k];


				//speed
					updateagents[i].d_objective[k] += 10*2*(updateagents[i].velocity.x*updateagents[i].d_velocity[k].x+updateagents[i].velocity.y*updateagents[i].d_velocity[k].y);
				
				//acceleration
				updateagents[i].d_objective[k] += 2*(updateagents[i].acceleration.x*updateagents[i].d_acceleration[k].x + updateagents[i].acceleration.y*updateagents[i].d_acceleration[k].y);

				//seek target
				if (dseeksq > .01)
					updateagents[i].d_objective[k] += ((updateagents[i].position.x - seekPos.x)*updateagents[i].d_position[k].x + (updateagents[i].position.y - seekPos.y)*updateagents[i].d_position[k].y)/sqrt(dseeksq);
				
				
				d_fobj[k] += updateagents[i].d_objective[k];
			}
		}
	}

	//swap the agent buffers
	agenttype *tptr;
	tptr = agents;
	agents = updateagents;
	updateagents = tptr;

	step++;
#ifdef PDBG
//Sleep(100);
	//	getchar();
#endif

	return flockOb;
}

void Flock::Reset()
{
	time  =0;
	step = 0;
	cells.clear();
	cells.resize(cellsX*cellsY);



	//reset the agents using the same random params
	memset(updateagents,0,numAgents*sizeof(agenttype));
	memset(agents,0,numAgents*sizeof(agenttype));
	for(int i = 0; i < numAgents;i++)
	{
		agents[i].position.x = gsl_ran_gaussian(rng, psigma) + toroidX/2;
		agents[i].position.y = gsl_ran_gaussian(rng, psigma) + toroidY/2;

		agents[i].position.x = (fmodf(agents[i].position.x,toroidX));
		agents[i].position.y = (fmodf(agents[i].position.y,toroidY));

		if (agents[i].position.x < 0)
			agents[i].position.x += toroidX;
		if (agents[i].position.y < 0)
			agents[i].position.y += toroidY;

		agents[i].angle = gsl_ran_gaussian(rng,asigma);
		agents[i].speed = 10 + gsl_ran_gaussian(rng,ssigma);

		agents[i].velocity.x = cosf(agents[i].angle)*agents[i].speed;
		agents[i].velocity.y = sinf(agents[i].angle)*agents[i].speed;

		agents[i].cell= floorf(agents[i].position.x / floorf(toroidX / cellsX)) + cellsX* floorf(agents[i].position.y / floorf(toroidY / cellsY));
		cells.at(agents[i].cell).insert(i);
	}
}



void Flock::Init(int nagents, double s, double a, double p)
{


	ssigma = s;
	asigma = a;
	psigma = p;

	time = 0;
	learning = true;
	numAgents = nagents;
	free(agents);
	free(updateagents);

	numObstacles = 0;
	obstacles = (obstype*)malloc(sizeof(obstype)*numObstacles);
	for (int i = 0; i < numObstacles;i++)
	{
		obstacles[i].position.x = gsl_ran_gaussian(rng, toroidX/3) + toroidX/2;
		obstacles[i].position.y = gsl_ran_gaussian(rng, toroidY/3) + toroidY/2;
		obstacles[i].position.x = (fmodf(obstacles[i].position.x,toroidX));
		obstacles[i].position.y = (fmodf(obstacles[i].position.y,toroidY));

		if (obstacles[i].position.x < 0)
			obstacles[i].position.x += toroidX;
		if (obstacles[i].position.y < 0)
			obstacles[i].position.y += toroidY;

		obstacles[i].radius = max(2,fabsf(2 + gsl_ran_gaussian(rng,.5)));

		obstacles[i].mincellX = floorf((obstacles[i].position.x-obstacles[i].radius-nradius) / floorf(toroidX / cellsX));
		obstacles[i].mincellY = floorf((obstacles[i].position.y-obstacles[i].radius-nradius) / floorf(toroidY / cellsY));

		obstacles[i].maxcellX = floorf((obstacles[i].position.x+obstacles[i].radius+nradius) / floorf(toroidX / cellsX));
		obstacles[i].maxcellY = floorf((obstacles[i].position.y+obstacles[i].radius+nradius) / floorf(toroidY / cellsY));

	}

	step = 0;

	seekPos.x = 600;
	seekPos.y = 500;

	//initialize the agents
	agents = (agenttype*)malloc(numAgents*sizeof(agenttype));
	updateagents = (agenttype*)malloc(numAgents*sizeof(agenttype));
	memset(agents,0,numAgents*sizeof(agenttype));
	int sz = numAgents*sizeof(agenttype);
	memset(updateagents,0,numAgents*sizeof(agenttype));

	for(i = 0; i < numAgents;i++)
	{
//		memset(&agents[i],0,sizeof(agents[i]));

		agents[i].position.x = gsl_ran_gaussian(rng, psigma) + toroidX/2;
		agents[i].position.y = gsl_ran_gaussian(rng, psigma) + toroidY/2;

		agents[i].position.x = (fmodf(agents[i].position.x,toroidX));
		agents[i].position.y = (fmodf(agents[i].position.y,toroidY));

		if (agents[i].position.x < 0)
			agents[i].position.x += toroidX;
		if (agents[i].position.y < 0)
			agents[i].position.y += toroidY;

		agents[i].angle = gsl_ran_gaussian(rng,asigma);
		agents[i].speed = 10 + gsl_ran_gaussian(rng,ssigma);

		agents[i].velocity.x = cosf(agents[i].angle)*agents[i].speed;
		agents[i].velocity.y = sinf(agents[i].angle)*agents[i].speed;

		agents[i].cell= floorf(agents[i].position.x / floorf(toroidX / cellsX)) + cellsX* floorf(agents[i].position.y / floorf(toroidY / cellsY));
		cells.at(agents[i].cell).insert(i);

	}
}

void Flock::FreeStatic()
{
	glDeleteLists(agentDL,1);
	agentDL = 0;
}

void Flock::AllocStatic()
{
	if (agentDL != 0)
		FreeStatic();

	agentDL = glGenLists(1);
	glNewList(agentDL,GL_COMPILE);

	glBegin(GL_TRIANGLES);

	glNormal3f(.268f,.3578f,.089f);
	glVertex3f(0,.25f,0);
	glVertex3f(0,0,1.0f);
	glVertex3f(.5f,0,-.5f);

	glNormal3f(-.268f,.3578f,.089f);
	glVertex3f(0,.25f,0);
	glVertex3f(-.5f,0,-.5f);
	glVertex3f(0,0,1.0f);

	glNormal3f(.596f,.298f,-.745f);
	glVertex3f(0,.25f,0);
	glVertex3f(0,0,-.1f);
	glVertex3f(-.5f,0,-.5f);

	glNormal3f(-.596f,.298f,-.745f);
	glVertex3f(0,.25f,0);
	glVertex3f(.5f,0,-.5f);
	glVertex3f(0,0,-.1f);

	glNormal3f(0,-1.0f,0);
	glVertex3f(0,0,-.1f);
	glVertex3f(.5f,0,-.5f);
	glVertex3f(0,0,1.0f);

	glVertex3f(0,0,-.1f);
	glVertex3f(0,0,1.0f);
	glVertex3f(-.5f,0,-.5f);
	glEnd();

	glEndList();

}
double Flock::GetTime()
{
	return time;
}

