#include <Repetoire.h>
#include <tinyxml.h>
#include <KeyFrameAnim.h>
#include <iostream>
#include <valarray>

std::map<std::string,Repetoire*> Repetoire::reps;

typedef  struct
{
	std::string name;
	std::map<std::string,float> attribs;
	std::string action;
} animstruct;

void printvalarrf(const std::valarray<float>& v)
{
	printf ("[");
	for(size_t i = 0; i < v.size(); i++)
	{
		printf("%.2f",v[i]);
		if (i < v.size() -1)
			printf(",");
	}
	printf ("]");
}

void printvalarri(const std::valarray<int>& v)
{
	printf ("[");
	for(size_t i = 0; i < v.size(); i++)
	{
		printf("%d",v[i]);
		if (i < v.size() -1)
			printf(",");
	}
	printf ("]");
}

Repetoire::Repetoire()
{
}

Repetoire::~Repetoire()
{
}
Repetoire * 
Repetoire::GetByName(const char * name)
{
	std::map<std::string, Repetoire*>::iterator i = reps.find(name);
	if (i == reps.end())
		return NULL;
	else
		return i->second;
}

bool 
Repetoire::LoadRepetoire(const char * ID, const char * fname)
{
	std::map<std::string,Repetoire*>::iterator i = reps.find(ID);
	if (i != reps.end())
	{
		if (i->second->filename == std::string(fname))
			return true;
		else
		{
			printf("Tried to load different repetoires under the same ID\n");
			return false;
		}
	}
	
	
	Repetoire * r = new Repetoire;

	if (!r->LoadXML(fname))
	{
		delete r;
		return false;
	}
	else
	{
		reps[ID] = r;
		return true;
	}
	

}

bool 
Repetoire::LoadXML( const char * name)
{

	//std::vector<animstruct> anims;
	filename = name;
	TiXmlDocument doc;
	if (!doc.LoadFile(name))
	{
		printf("Error loading actor file: %s\n",doc.ErrorDesc());
		return false;
	}
	
	TiXmlNode* node = 0;
	TiXmlHandle docHandle( &doc );
	
	TiXmlElement* animEl = docHandle.FirstChild( "repetoire" ).FirstChild( "animations" ).Element();

	if (!animEl)
	{
		printf("Bad XML format - no animations node\n");
		return false;
	}

	//iterate through each animation and add it to the applicable actionset
	for( TiXmlElement* child = animEl->FirstChildElement("anim"); child; child=child->NextSiblingElement() )
	{
		std::map<std::string,float> attribs;

		if (!KeyFrameAnim::Load(child->Attribute("name"),child->Attribute("name")))
			return false;
		
		//iterate through this animation's attributes
		for( TiXmlElement* att = child->FirstChildElement("att"); att; att=att->NextSiblingElement("att") )
		{
			const char * name = att->Attribute("s");
			double val;
			if (!name)
			{
				printf ("Malformed animation %s\n",child->Attribute("name"));
				return false;
			}
			if (att->QueryDoubleAttribute("w",&val) != TIXML_SUCCESS || val < 0.0f)
				val = 1.0f;

			attribs[name] = (float)val;
		}

		TiXmlElement* act = child->FirstChildElement("action");
		if (!act)
		{
			printf ("Malformed animation %s: no action node",child->Attribute("name"));
			return false;
		}

		const char * actionname = act->Attribute("s");
		if (!actionname)
		{
			printf ("Malformed animation %s : no action name\n",child->Attribute("name"));
			return false;
		}

		//now add this animation to the appropriate actionset
		std::map<std::string, ActionSet>::iterator i = actionsets.find(actionname);
		if (i == actionsets.end())
			actionsets[actionname] = ActionSet(actionname);

		actionsets[actionname].AddAnim(std::string(child->Attribute("name")),attribs);
		attribs.clear();

	}

/*
	//build the hard and soft matrices
	hard.resize(actions.size()*anims.size(),0);
	soft.resize(attributes.size()*anims.size(),0.0f);

	for (unsigned int i = 0; i < anims.size(); i++)
	{
		for(std::set<std::string>::iterator j = anims[i].actions.begin(); j != anims[i].actions.end(); j++)
		{
			hard[i*actions.size()+ actions[*j]] = 1;
		}
		
		for(std::map<std::string,float>::iterator j = anims[i].attribs.begin(); j != anims[i].attribs.end(); j++)
		{
			soft[i*attributes.size()+ attributes[j->first]] = j->second;
		}
	}

	for (unsigned int i = 0; i < anims.size(); i++)
	{
		std::slice s(i*attributes.size(),attributes.size(),1);
		printvalarrf(soft[s]);
		printf("\t");
		s = std::slice(i*actions.size(),actions.size(),1);
		printvalarri(hard[s]);
		printf("\n");

	}

	for (std::map<std::string,int>::iterator i = attributes.begin(); i != attributes.end(); i++)
	{
		printf("%d %s\n",i->second,i->first.c_str());
	}

*/
	return true;
}


std::string 
Repetoire::PickAnim(const std::string & action, const std::map<std::string,float> & softc)
{
	std::map<std::string, ActionSet>::iterator i = actionsets.find(action);
	if (i == actionsets.end())
		return "";

	return i->second.PickAnim(softc);
	
}
